﻿using JetBrains.Annotations;

namespace Magify
{
    public class MagifyConfig
    {
        /// <summary>
        /// Magify application name for iOS
        /// </summary>
        public string AppNameIOS { get; init; }

        /// <summary>
        /// Path in StreamingAssets to default application config json for current version for iOS
        /// </summary>
        public string ConfigPathIOS { get; init; }

        /// <summary>
        /// Magify application name for Android
        /// </summary>
        public string AppNameGP { get; init; }

        /// <summary>
        /// Path in StreamingAssets to default application config json for current version for Android
        /// </summary>
        public string ConfigPathGP { get; init; }

        /// <summary>
        /// SDK data storage path
        /// </summary>
        internal string StoragePath { get; init; } = PackageInfo.PersistentPath;

        /// <summary>
        /// Custom client id. Make sure to use same client id for all services (AppsFlyer, Adjust, Applovin and etc.)
        /// </summary>
        public string CustomClientId { get; init; }

        /// <summary>
        /// Use it for emulating campaigns in editor
        /// </summary>
        public MagifyEditorMoq EditorMoq { get; init; }

        /// <summary>
        /// If you want to use the full functionality of the Magify SDK, use this flag
        /// </summary>
        public bool UseAdvancedVersion { get; init; } = false;

        /// <summary>
        /// If you want to use only moq use this flag
        /// </summary>
        public bool IsDeveloperMode { get; init; }

        /// <summary>
        /// Use it for emulating device in editor
        /// </summary>
        public EditorDevice EditorDevice { get; init; }

        /// <summary>
        /// Settings for Magify presenter canvas
        /// </summary>
        [CanBeNull]
        public IPresenterSettings PresenterSettings { get; init; }

        /// <summary>
        /// Default environment. You can change it later using <see cref="MagifyManager.Environment"/>
        /// </summary>
        public Environment Environment { get; init; } = Environment.Production;

        /// <summary>
        /// Default SubscriptionStatus. You can change it later using <see cref="MagifyManager.SubscriptionStatus"/>
        /// </summary>
        public SubscriptionStatus SubscriptionStatus { get; init; } = SubscriptionStatus.Inactive;

        /// <summary>
        /// Default AuthorizationStatus. You can change it later using <see cref="MagifyManager.AuthorizationStatus"/>
        /// </summary>
        public AuthorizationStatus AuthorizationStatus { get; set; } = AuthorizationStatus.Unauthorized;

        /// <summary>
        /// State of user consent to GDPR
        /// </summary>
        /// <remarks>If the value is null, default value(false) or saved value from previous session will be used.</remarks>
        [CanBeNull]
        public bool? IsGdprApplied { get; set; }

        /// <summary>
        /// State of user consent to ATT
        /// </summary>
        /// <remarks>If the value is null, default value(false) or saved value from previous session will be used.</remarks>
        [CanBeNull]
        public bool? IsAttAuthorized { get; set; }

        /// <summary>
        /// Default logging status. You can change it later using <see cref="MagifyManager.IsLoggingEnabled"/>
        /// </summary>
        public bool IsLoggingEnabled { get; init; } = true;

        public string RevenuePerCountryPath { get; init; }
        public string RevenueLevelsPath { get; init; }
        public string DefaultCurrencyRatesPath { get; init; }
        public string SubscriptionMultipliersPath { get; init; }
    }
}