﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using JetBrains.Annotations;
using UniRx;

namespace Magify
{
    internal interface IMagifyClient : ICancelable
    {
        internal const string CountersFolderName = "counters";
        internal const string GeneralPrefsName = "general.bin";
        internal const string OffersPrefsName = "lto.bin";

        IFeatures Features { get; }
        IReactiveProperty<bool> IsGeoIpEnabled { get; }
        IReactiveProperty<Environment> Environment { get; }
        [NotNull]
        public IReactiveProperty<IPurchaseVerificationHandler> ExternalPurchaseVerificationHandler { get; }
        public FloatReactiveProperty VerificationRetryInterval { get; }
        IReactiveProperty<SubscriptionStatus> SubscriptionStatus { get; }
        IReactiveProperty<InAppStatus> InAppStatus { get; }
        IReactiveProperty<AuthorizationStatus> AuthorizationStatus { get; }
        IReactiveProperty<string> AdjustId { get; }
        IReactiveProperty<string> FirebaseInstanceId { get; }
        IReactiveProperty<string> ReferrerId { get; }
        IReactiveProperty<bool> IsGdprGranted { get; }
        IReadOnlyReactiveProperty<string> GpsAdId { get; }
        IReadOnlyReactiveProperty<bool?> AttAuthorizationStatus { get; }
        bool IsTablet { get; }
        bool IsPortrait { get; }
        ContextSyncTime LastContextSyncTime { get; }
        IReactiveProperty<TimeSpan> SessionsInterval { get; }
        string ClientId { get; }
        LimitsModel Limits { get; }
        IReadOnlyReactiveProperty<int> SessionNumber { get; }
        List<InAppProduct> InAppProducts { get; }
        List<SubscriptionProduct> SubscriptionProducts { get; }
        PlatformAPI Platform { get; }
        string FirstInstalledVersion { get; }
        event Action OnConfigLoaded;
        event Action OnConfigSyncRequested;
        event Action<LtoInfo> OnOfferAdded;
        event Action<LtoInfo> OnOfferUpdated;
        event Action<LtoInfo> OnOfferRemoved;
        event Action<LtoInfo> OnOfferFinished;
        event Action OnPurchasedProductsChanged;
        event Action OnUserDidTakeScreenshot;
        event Action OnApplicationEnterForeground;
        event Action OnApplicationEnterBackground;
        void InitializeSdk();
        void Setup();
        void Sync();
        ICampaign CampaignFor(string eventName, Dictionary<string, object> customParams = null, bool silent = false);
        bool IsCampaignAvailable(string campaignName, string eventName, Dictionary<string, object> customParams = null);
        IReadOnlyCollection<LtoInfo> GetActiveLtoOffers();
        void CompleteOffer(string campaignName);
        CampaignImpression GetCampaignImpression(string campaignName);
        List<ContentItem> GetContentList(string group, string key, ICollection<string> tags = null);
        ContentItem GetEarliestContent(string group, string key);
        ContentItem GetLatestContent(string group, string key);
        bool HasProcessedPurchase(string productId);
        bool SubscribeCampaignUpdates(ICampaign campaign, Action onUpdate);
        void UnsubscribeCampaignUpdates(string campaignName, Action onUpdate);
        void ClearSubscriptions(string campaignName);
        void TweakAnalyticsConfig(int eventsGroupSize, int syncTimeInterval);
        void ResetAnalyticsConfig();
        Task<string> ForceReissueAuthToken(CancellationToken cancellationToken = default);
        void RefreshGpsAdId();
        void TweakUserLocale(string languageTag);
        void SetAttStatus(bool authorized);
        void SetGameLevel(int level);
        void SetGameMaxLevel(int maxLevel);
        void SetGameMode(string mode);
        void SetUserEmail(string email);
        void SetMediaSource(string networkName, string campaignName, string adGroup);
        void SetupConversionTracker(string revenuePerCountryPath, string revenueLevelsPath, string defaultCurrencyRatesPath, string subscriptionMultipliersPath);
        void TrackCustomEvent(string eventName, Dictionary<string, object> customParams = null);
        void TrackImpression(string campaignName);
        void TrackParentCampaignImpression(string campaignName);
        void TrackProductsImpression(string campaignName, IReadOnlyList<string> productIds);
        void TrackImpressionFailFor(string campaignName, string reason);
        void TrackAdsImpression(string campaignName, IAdsImpression impression);
        void TrackClickFor(string campaignName);
        void TrackProductClickFor(string campaignName, string productId);
        void TrackAdsClickFor(string campaignName);
        void TrackAdsProductClickFor(string campaignName, string productId);
        void TrackSubscriptionActivation(bool isTrial, string productId, string price, string currency, string transactionId, string purchaseToken, string originalTransactionId, string receipt);
        void TrackExternalSubscriptionActivation(bool isTrial, string productId, string price, string currency, string transactionId, string purchaseToken, string originalTransactionId, string receipt);
        void TrackInAppFor(string productId, string price, string currency, string transactionId, string purchaseToken, string originalTransactionId, string receipt);
        void TrackExternalInAppFor(string productId, string price, string currency, string transactionId, string purchaseToken, string originalTransactionId, string receipt);
        void TrackRestoredInAppFor(string productId);
        void TrackRestoredSubscription(string productId);
        void TrackRewardGranted(string productId);
        void TrackFreeBonusGranted(string productId);
        void TrackOrdinaryProductUsed(string productId);
        void TrackIncomeTransaction(string source, List<BonusInfo> bonuses, ProductInfo product = null);
        void TrackExpenseTransaction(List<BonusInfo> bonuses);
        void TrackCorrectionTransaction(List<BonusInfo> bonuses);
        CampaignRequest GetLastCampaignRequest(string campaignName);
        CampaignRequest GetLastCampaignRequest(CampaignType campaignType);
        void ResetContext(bool cancelIfLoading);
        void Reset();
        void ClearNativeStorage();
    }
}