﻿using JetBrains.Annotations;

namespace Magify
{
    public readonly struct EndpointUrl
    {
        public static readonly EndpointUrl Production = new("https://api.middle-earth.io");
        public static readonly EndpointUrl Staging = new("https://eagle.jx-staging.middle-earth.io");
        public static readonly EndpointUrl Offline = new(null);

        public readonly string Value;
        public bool IsOffline => string.IsNullOrEmpty(Value);

        public EndpointUrl([CanBeNull] string value)
        {
            Value = value;
        }

        public static bool operator ==(EndpointUrl a, EndpointUrl b) => a.Value == b.Value;

        public static bool operator !=(EndpointUrl a, EndpointUrl b) => !(a == b);

        private bool Equals(EndpointUrl other) => Value == other.Value;

        public override bool Equals(object obj) => obj is EndpointUrl other && Equals(other);

        public override int GetHashCode() => Value.GetHashCode();

        internal static EndpointUrl FromEnvironment(Magify.Environment environment)
        {
            return environment switch
            {
                Magify.Environment.Production => EndpointUrl.Production,
                Magify.Environment.Staging => EndpointUrl.Staging,
                Magify.Environment.Offline => EndpointUrl.Offline,
                _ => throw new InvalidMagifyEnvironmentException(environment)
            };
        }
    }
}