﻿using System;
using FluentAssertions;
using Magify.Model;
using NUnit.Framework;
using UniRx;

namespace Magify.Tests
{
    internal partial class MagifyClientTests
    {
        public class AppLaunch
        {
            [Test]
            public void WhenMagifyClientInitialized_ThanOnlyOneAppLaunchEventSent()
            {
                // Arrange
                var appLaunchEvents = 0;
                using var client = CreateDefault();
                GetAnalyticsTracker(client)
                    .OnEventSent
                    .Where(e => e is EventType.AppLaunch)
                    .Subscribe(_ => appLaunchEvents++);

                // Act
                client.InitializeSdk();

                // Assert
                appLaunchEvents.Should().Be(1);
            }

            [Test]
            [TestCase(1)]
            [TestCase(2)]
            [TestCase(7)]
            [TestCase(30)]
            public void WhenNewSessionStarted_ThanOnlyOneAppLaunchEventSent(int backgroundForegroundCycles)
            {
                // Arrange
                var appLaunchEvents = 0;
                using var client = CreateDefaultAndInit();
                var platform = (PlatformDefault)GetPlatformAPI(client);
                GetAnalyticsTracker(client)
                    .OnEventSent
                    .Where(e => e is EventType.AppLaunch)
                    .Subscribe(_ => appLaunchEvents++);

                // Act
                // Simulate session change
                GetSessionCounter(client).Interval.Value = TimeSpan.Zero;
                for (var i = 0; i < backgroundForegroundCycles; i++)
                {
                    platform.GoToBackground();
                    platform.ReturnToForeground();
                }

                // Assert
                appLaunchEvents.Should().Be(backgroundForegroundCycles);
            }
        }
    }
}